/*
 * Copyright (c) 2009, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>
#include <netinet/in.h>

#include <cmdline_parse_ipaddr.h>
#include <cmdline_parse.h>

#include "parse_obj_list.h"

/* This file is an example of extension of libcmdline. It provides an
 * example of named objects stored in a list, supporting the
 * completion on objects name */

static int
parse_obj_list(cmdline_parse_token_hdr_t *tk, const char *buf, void *res,
	       unsigned ressize)
{
	struct token_obj_list *tk2 = (struct token_obj_list *)tk;
	struct token_obj_list_data *tkd = &tk2->obj_list_data;
	struct object *o;
	unsigned int token_len = 0;

	if (res && ressize < sizeof(struct object *))
		return -1;

	token_len = strlen(buf);

	SLIST_FOREACH(o, tkd->list, next) {
		if (strcmp(buf, o->name))
			continue;
		break;
	}
	if (o == NULL) /* not found */
		return -1;

	/* store the address of object in structure */
	if (res)
		*(struct object **)res = o;

	return 0;
}

static int
complete_obj_list_start(cmdline_parse_token_hdr_t *tk,
			__attribute__((unused)) const char *tokstr,
			void **opaque)
{
	struct token_obj_list *tk2 = (struct token_obj_list *)tk;
	struct token_obj_list_data *tkd = &tk2->obj_list_data;
	struct object *o;

	o = SLIST_FIRST(tkd->list);
	*opaque = (void *)o;
	if (o == NULL)
		return -1; /* no completion */
	return 0;
}

static int
complete_obj_list_iterate(cmdline_parse_token_hdr_t *tk, void **opaque,
			  char *dstbuf, unsigned int size)
{
	struct object *o;
	int len;

	o = *opaque;
	if (o == NULL)
		return -1;

	len = snprintf(dstbuf, size, "%s", o->name);
	if (len < 0 || len >= size)
		return -1;

	strcpy(dstbuf, o->name);
	o = SLIST_NEXT(o, next);
	*opaque = o;
	return 0;
}


static int
cmdline_help_obj_list(cmdline_parse_token_hdr_t *tk, char *dstbuf,
		      unsigned int size)
{
	snprintf(dstbuf, size, "Obj-List");
	return 0;
}

struct cmdline_token_ops token_obj_list_ops = {
	.parse = parse_obj_list,
	.complete_start = complete_obj_list_start,
	.complete_iterate = complete_obj_list_iterate,
	.complete_end = NULL,
	.help = cmdline_help_obj_list,
};
