#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/queue.h>

#include "expression.h"
#include "conf_parser.h"
#include "confnode.h"

#define HASHTABLE_ORDER 10
#define HASHTABLE_SIZE  (1<<HASHTABLE_ORDER)
#define HASHTABLE_MASK  (HASHTABLE_ORDER-1)

static struct confnode_list conf_htable[HASHTABLE_SIZE];

/* return the hash from name */
static inline uint32_t hash_name(const char *name)
{
	uint32_t h = 0;
	const unsigned char *c;

	for (c = (const unsigned char *)name; *c; c++)
		h += ((((uint32_t)*c) << 4) + (((uint32_t)*c) >> 4)) * 11;

	return h & HASHTABLE_MASK;
}

/*
 * Return the configuration node associated to the given name. If not
 * found, return NULL.
 */
struct confnode *conf_htable_lookup(const char *name)
{
	struct confnode *c;
	uint32_t h = hash_name(name);

	TAILQ_FOREACH(c, &conf_htable[h], hnext) {
		if (!strncmp(name, c->name, sizeof(c->name)))
			break;
	}
	return c;
}

/*
 * Parse a configuration tree and add all the nodes in the hashtable.
 */
int conf_htable_fill(struct confnode *root)
{
	struct confnode *c;
	uint32_t h = hash_name(root->name);

	TAILQ_INSERT_TAIL(&conf_htable[h], root, hnext);

	TAILQ_FOREACH(c, &root->children, next) {
		conf_htable_fill(c);
	}
	return 0;
}

/*
 * Initialize the htable to be empty.
 */
int conf_htable_init(void)
{
	unsigned i;

	for (i=0; i<HASHTABLE_SIZE; i++) {
		TAILQ_INIT(&conf_htable[i]);
	}
	return 0;
}

