/*
 * Copyright (c) 2010, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/queue.h>

#include "expression.h"
#include "conf_parser.h"
#include "conf_htable.h"
#include "parser_common.h"
#include "dotconfig.h"
#include "confnode.h"

static int confnode_root_new(struct confnode *n,
			     const struct line *line);

static struct confnode_type confnode_root = {
	.ops = {
		.new = confnode_root_new,
		.free = NULL,
		.add_attr = NULL,
		.close_dir = NULL,
		.dotconfig_write = NULL,
		.strvalue_to_boolvalue = NULL,
		.set_user_strvalue = NULL,
		.get_user_strvalue = NULL,
		.set_default_strvalue = NULL,
		.get_default_strvalue = NULL,
		.get_type_str = NULL,
		.display_short = NULL,
		.display_long = NULL,
	}
};

/* Create a new node */
static int confnode_root_new(struct confnode *n, const struct line *line)
{
	struct token *tok;
	tok = TAILQ_FIRST(&line->token_list);

	if (strcmp(tok->str, "root") || line->n_token != 1)
		return -1;

	snprintf(n->name, sizeof(n->name), "root");
	n->ops = &confnode_root.ops;
	n->flags = CONFNODE_F_IS_ROOT;
	return 0;
}

/* register the node type */
void confnode_root_register(void)
{
	TAILQ_INSERT_TAIL(&confnode_type_list, &confnode_root, next);
}
