#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/queue.h>

#include "expression.h"
#include "parser_common.h"
#include "conf_parser.h"
#include "confnode.h"
#include "conf_htable.h"

int dotconfig_parse_line(const char *line)
{
	struct confnode *opt;
	char buf[BUFSIZ];
	char *nodename, *s;
	const char *optval;
	char wait_char = '=';

	/* copy line in a writable buffer */
	buf[sizeof(buf) - 1] = '\0';
	strncpy(buf, line, sizeof(buf) - 1);
	s = buf;

	while (*s != '\0' && isspace(*s))
		s++;
	/* nothing to do, empty line */
	if (*s == '\0')
		return 0;

	/* skip the first "#", expecting "is not set" */
	if (*s == '#') {
		wait_char = ' ';
		s++;
		while (*s != '\0' && isspace(*s))
			s++;
		/* nothing to do, empty line */
		if (*s == '\0')
			return 0;
	}

	/* get node name, ignore invalid names */
	nodename = s;
	if (strncmp("CONFIG_", nodename, strlen("CONFIG_")))
		return 0;
	nodename += strlen("CONFIG_");

	while (*s != '\0' && *s != wait_char)
		s++;
	/* invalid line */
	if (*s != wait_char)
		return -1;

	*s = '\0';

	/* find the node in the conf htable */
	opt = conf_htable_lookup(nodename);
	if (opt == NULL) {
		printf("ignore unknown symbol <%s>\n", nodename);
		return 0;
	}

	s++;

	/* "is not set" -> "n" */
	if (wait_char == ' ') {
		if (strcmp(s, "is not set\n"))
			return 0;
		optval = "n";
	}
	else {
		if (*s == '"')
			s ++;
		optval = s;
		/* erase the cariage return */
		while (*s != '\0' && *s != '\n') {
			if (*s == '"' && *(s-1) != '\\') {
				*s = '\0';
				break;
			}
			s++;
		}
		*s = '\0';
	}

	/* set its value in the conf tree */
	if (confnode_set_user_strvalue(opt, optval) < 0) {
		printf("cannot assign value to node <%s>: %s\n", nodename, optval);
		return -1;
	}

	return 0;
}

/* parse .config */
int dotconfig_read(const char *filename, struct confnode *conf)
{
	FILE *f;
	char buf[BUFSIZ];
	int err = 0;

	f = fopen(filename, "r");
	if (f == NULL) {
		printf("cannot find file <%s>\n", filename);
		return -1;
	}
	while (fgets(buf, sizeof(buf), f) != NULL) {

		if (dotconfig_parse_line(buf) < 0) {
			err = -1;
			break;
		}
	}
	fclose(f);

	return err;
}

/* write .config */
int dotconfig_write(const char *filename, const struct confnode *conf)
{
	FILE *f;
	const struct confnode *c;

	f = fopen(filename, "w");
	if (f == NULL) {
		printf("cannot open file <%s>\n", filename);
		return -1;
	}

	/* dump all the children of root node */
	TAILQ_FOREACH(c, &conf->children, next) {
		confnode_dotconfig_write(c, f);
	}

	fclose(f);

	return 0;
}

